function [stopNow,parameters]=setParameters(nargout,list_)
% [stopNow,parameters]=setParameters(varargin)
%
% Assigns a list of parameters in the caller's workspace. 'list'
% is a cell array of the form
% {'variable name 1',value 1,'variable name 2', value 2,...}
%
% All parameters assigned are also returned as fields in a structure:
%   parameters.{variable name}
%
% A parameter list with a single value {'help'} results in the
% output of a help header
%
% returns stopNow=true if the function should return immediatly returning
% 'parameters' as the 1st argument -- typically when the function is called with help
%
% This function is typically used within a m-script function as follows:
%
% function [varargout]=scriptName(varargin)
% % For help on the input parameters type 'scriptName Help' 
%
% % Function global help
% declareParameter(...
%     'Help', { '...' })
%
% % Declare all input parameters, see 'help declareParameter'
% declareParameter( .... ); 
%
% % Retrieve parameters and inputs
% [stopNow,parameters]=setParameters(nargout,varargin);
% if stopNow
%    return;
% end
%
% % Start main code here
%
%   ....
% 
% % Set outputs
% 
% vargout=setOutputs(nargout,parameters);
%
% Copyright (C) 2010-2014  Joao Hespanha

verboseLevel=0;  % 0 none, 1 less, 2 more

callerName_=dbstack(1);
callerName_=callerName_(1).name;

parameters=struct();

%% Get localVariables_ from caller's workspace

if evalin('caller','exist(''localVariables_'',''var'')')
    callerVariables=evalin('caller','localVariables_');
else
    error('setParameters(%s): Use declareParameters before calling setParameter\n',callerName_);
end

%% Print help and exit with stopNow=1
if length(list_)==1 && strcmp(lower(list_{1}),'help')
    if nargout==0
        %% Print help and exit with stopNow=1
        % print template
        fprintf('%% [...]=%s(''parameter name 1'',value,''parameter name 2'',value,...);\n%%\n',callerName_);
        % print function description
        Help_=0;
        for i_=1:length(callerVariables)
            if strcmp(callerVariables{i_}.VariableName,'Help_')
                Help_=1;
                for j_=1:length(callerVariables{i_}.Description)
                    fprintf('%% %s\n',callerVariables{i_}.Description{j_});
                end
            end
        end    
        if Help_
            fprintf('%%\n');
        end
        % print input parameters
        first=1;
        for i_=1:length(callerVariables)
            if ~strcmp(callerVariables{i_}.VariableName,'Help_') && ...
                    strcmp(callerVariables{i_}.type,'parameter')
                if first
                    fprintf('%% Input parameters:\n%% ----------------\n');
                    first=0;
                end
                fprintf('%%\n%% %s',callerVariables{i_}.VariableName);
                if isfield(callerVariables{i_},'DefaultValue') 
                    fprintf(' [default %s]',value2str(callerVariables{i_}.DefaultValue));
                end
                if isfield(callerVariables{i_},'AdmissibleValues') 
                    fprintf('\n%%    in [');
                    for j_=1:length(callerVariables{i_}.AdmissibleValues)
                        fprintf('%s',value2str(callerVariables{i_}.AdmissibleValues{j_}));
                        if j_<length(callerVariables{i_}.AdmissibleValues)
                            fprintf(',');
                        else
                            fprintf(']');
                        end
                    end
                end
                fprintf('\n');
                for j_=1:length(callerVariables{i_}.Description)
                    fprintf('%%    %s\n',callerVariables{i_}.Description{j_});
                end
                if isfield(callerVariables{i_},'PossibleInheritances') 
                    fprintf('%%\n%%    Typically obtained from:');
                    for l_=1:size(callerVariables{i_}.PossibleInheritances,1)
                        script=sprintf('%s(%s)',callerVariables{i_}.PossibleInheritances{l_,1:2});
                        if isempty(callerVariables{i_}.PossibleInheritances{l_,3})&&isempty(callerVariables{i_}.PossibleInheritances{l_,3})
                            fprintf('\n%%       %s',script);
                        else
                            fprintf('\n%%       regexprep(%s,''%s'',''%s'')',...
                                    script,callerVariables{i_}.PossibleInheritances{l_,3:4});
                        end
                        if l_<size(callerVariables{i_}.PossibleInheritances,1)
                            fprintf(' or');
                        end
                    end
                    fprintf('\n');
                end
            end
        end
        % print Outputs
        first=1;
        for i_=1:length(callerVariables)
            if ~strcmp(callerVariables{i_}.VariableName,'Help_') && ...
                    strcmp(callerVariables{i_}.type,'output')
                if first
                    fprintf('%%\n%% Outputs:\n%% -------\n');
                    first=0;
                end
                fprintf('%%\n%% %s',callerVariables{i_}.VariableName);
                fprintf('\n');
                for j_=1:length(callerVariables{i_}.Description)
                    fprintf('%%    %s\n',callerVariables{i_}.Description{j_});
                end
            end
        end
        fprintf('%%\n\n');
        %error('setParameters(%s): Called with ''help'' -- no error\n',callerName_);
        parameters=NaN;
        stopNow=true;
        return;
    else
        parameters=callerVariables;
        assignin('caller','varargout',{parameters});
        stopNow=true;
        return
    end
end
stopNow=0;

if mod(length(list_),2)==1
    error('setParameters(%s): Length of ''list'' must be even (%d instead)\n',callerName_,length(list_));
end

if verboseLevel>=2
    fprintf('setParameters(%s): Setting  parameters for %s(%d parameters);\n',callerName_,callerName_,length(list_)/2);
end

%% Assign values from parameter 'list'
for j_=1:2:length(list_)
    assigned=0;
    for i_=1:length(callerVariables)
        if  strcmp(callerVariables{i_}.type,'output')
            continue;
        end
        if strcmp(list_{j_},callerVariables{i_}.VariableName)
            if verboseLevel>=2
                fprintf('setParameters(%s): Explicit assignment ''%s''\n',callerName_,list_{j_})
            end
            assignin('caller',list_{j_},list_{j_+1});
            parameters=setfield(parameters,list_{j_},list_{j_+1});
            assigned=1;
            break
        end
    end    
    if ~assigned
        error('setParameters(%s): Undeclared parameter ''%s''\n',callerName_,list_{j_});
    end
end

%% Assign values from parametersStructure
% get value for the variable 'parametersStructure'
% (either from caller's workspace or from default value)
if evalin('caller','exist(''parametersStructure'',''var'')')
    parametersStructure=evalin('caller','parametersStructure');
else
    if verboseLevel>=2
        fprintf('setParameters(%s): Undefined ''parametersStructure'', looking for default value\n',...
                callerName_);
    end
    % look for default value
    for i_=length(callerVariables):-1:1
        if strcmp(callerVariables{i_}.VariableName,'parametersStructure') && ...
                isfield(callerVariables{i_},'DefaultValue')
            if verboseLevel>=1
                fprintf('setParameters(%s): Default  assignment %s=%s\n',...
                        callerName_,callerVariables{i_}.VariableName,...
                        value2str(callerVariables{i_}.DefaultValue));
            end
            assignin('caller',callerVariables{i_}.VariableName,...
                              callerVariables{i_}.DefaultValue); 
            parameters=setfield(parameters,callerVariables{i_}.VariableName,...
                                callerVariables{i_}.DefaultValue); 
            parametersStructure=callerVariables{i_}.DefaultValue;
            break;
        end
    end
end

% get values in 'parametersStructure'
if exist('parametersStructure','var')
    for i_=length(callerVariables):-1:1
        if  strcmp(callerVariables{i_}.type,'output')
            continue;
        end
        cmd=sprintf('~exist(''%s'',''var'')',callerVariables{i_}.VariableName);
        if evalin('caller',cmd)   % does not exist in caller's workspace?
            if isfield(parametersStructure,callerVariables{i_}.VariableName)
                value=getfield(parametersStructure,callerVariables{i_}.VariableName);
                parameters=setfield(parameters,callerVariables{i_}.VariableName,value);
                if verboseLevel>=1
                    fprintf('setParameters(%s): parametersStructure assignment %s=%s\n',...
                            callerName_,callerVariables{i_}.VariableName,...
                            value2str(value));
                end
                assignin('caller',callerVariables{i_}.VariableName,value);
            else
                if verboseLevel>=2
                    fprintf('setParameters(%s): Unassigned variable ''%s'' not in parametersStructure\n',...
                            callerName_,callerVariables{i_}.VariableName);
                end
            end
        end
    end
else
    if verboseLevel>=2
        fprintf('setParameters(%s): Undefined variable ''parametersStructure''\n',...
                callerName_);
    end
end

%% Assign default values
for i_=length(callerVariables):-1:1
    if isfield(callerVariables{i_},'DefaultValue') && ...
            ~strcmp(class(callerVariables{i_}.DefaultValue),'outputWithPedigree')
        cmd=sprintf('~exist(''%s'',''var'')',callerVariables{i_}.VariableName);
        if evalin('caller',cmd)  % does not exist in caller's workspace?
            if verboseLevel>=1
                fprintf('setParameters(%s): Default  assignment %s=%s\n',...
                        callerName_,callerVariables{i_}.VariableName,...
                        value2str(callerVariables{i_}.DefaultValue));
            end
            assignin('caller',callerVariables{i_}.VariableName,...
                              callerVariables{i_}.DefaultValue);
            parameters=setfield(parameters,callerVariables{i_}.VariableName,...
                              callerVariables{i_}.DefaultValue);
        end
    end
end 

%% look for DefaultValues from pedigree
if evalin('caller','exist(''pedigreeClass'',''var'')')
    pedigreeClass=evalin('caller','pedigreeClass');
else
    pedigreeClass='';
end

if ~isempty(pedigreeClass)
    [className,pedigreeName,...
     pedigreeSuffix,dateFormat,basenameUniqueRegexp,timeStampFormat,pedigreeWildcard,...
     reusingPedigree...
    ]=createPedigree(pedigreeClass,parameters,callerName_);
    assignin('caller','className_',className);
    assignin('caller','pedigreeName_',pedigreeName);
    for i_=length(callerVariables):-1:1
        if isfield(callerVariables{i_},'DefaultValue') && ...
                strcmp(class(callerVariables{i_}.DefaultValue),'outputWithPedigree')
            filename=sprintf('%s+%s',className,callerVariables{i_}.VariableName);
            assignin('caller',callerVariables{i_}.VariableName,filename);
        end
    end
else
    assignin('caller','className_','');
    assignin('caller','pedigreeName_','');
end

%% Check for admissible values
for i_=1:length(callerVariables)
    cmd=sprintf('exist(''%s'',''var'')',callerVariables{i_}.VariableName);
    if evalin('caller',cmd)  % exists in caller space?
        if isfield(callerVariables{i_},'AdmissibleValues')
            value=evalin('caller',callerVariables{i_}.VariableName);
            if checkAdmissible(value,callerVariables{i_}.AdmissibleValues)
                err=sprintf(['setParameters(%s): Unadmissible value %s=%s, not in\n   ['],...
                            callerName_,callerVariables{i_}.VariableName,value2str(value));
                for j_=1:length(callerVariables{i_}.AdmissibleValues)
                    err=[err,sprintf('%s',value2str(callerVariables{i_}.AdmissibleValues{j_}))];
                    if j_<length(callerVariables{i_}.AdmissibleValues)
                        err(end+1)=',';
                    else
                        err(end+1)=']';
                    end
                end
                error(err);
            end
        end
    else
        if ~strcmp(callerVariables{i_}.VariableName,'Help_') && ...
                ~strcmp(callerVariables{i_}.type,'output') 
            error('setParameters(%s): Variable ''%s'' has not been assigned\n',callerName_,callerVariables{i_}.VariableName);
        end
    end 
end

%% Check if script should be executed
if evalin('caller','exist(''executeScript'',''var'')')
    executeScript=evalin('caller','executeScript');
    if strcmp(executeScript,'yes') || (strcmp(executeScript,'asneeded') && ~reusingPedigree)
        stopNow=false;
    else
        if isempty(pedigreeClass)
            error('setParameters: executeScript=''false'' only allowed when ''pedigreeClass'' is defined\n');
        end
        stopNow=true;
        %% Assign output values
        vargout={};
        k=1;
        for i_=1:length(callerVariables)
            if strcmp(callerVariables{i_}.type,'output')
                filename=sprintf('%s+%s.mat',className,callerVariables{i_}.VariableName);
                vargout{k}=outputWithPedigree(pedigreeName,filename,callerVariables{i_}.VariableName);
                k=k+1;
            end
        end
        if nargout~=length(vargout)
            error('setParameters: %d outputs declared, but function was called with %d outputs\n',length(vargout),nargout);
        end
        assignin('caller','varargout',vargout);
    end
end

%% Get variables from  files
if ~stopNow
    for i_=1:length(callerVariables)
        if  strcmp(callerVariables{i_}.type,'output')
            continue;
        end
        cmd=sprintf('exist(''%s'',''var'') && strcmp(class(%s),''outputWithPedigree'')',...
                    callerVariables{i_}.VariableName,callerVariables{i_}.VariableName);
        if evalin('caller',cmd)  % exists in caller space?
            if verboseLevel>=2
                fprintf('setParameters(%s): outputWithPedigree found in ''%s''\n',...
                        callerName_,callerVariables{i_}.VariableName);
            end
            obj=evalin('caller',callerVariables{i_}.VariableName);
            assignin('caller',callerVariables{i_}.VariableName,getValue(obj,callerName_));
        end
    end    
end

%assignin('caller','parameters_',parameters);
    
function err=checkAdmissible(value,admissible)

err=1;
for i=1:length(admissible)
    if strcmp(value2str(value),value2str(admissible{i}))
        err=0;
        break
    end
end
